# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.cell_format import CellFormat
from ...types.create_new_sheet_tab_response import CreateNewSheetTabResponse
from ...types.get_table_response import GetTableResponse
from ...types.sheet import Sheet
from ...types.sheet_operation_response import SheetOperationResponse
from ...types.table_row_data import TableRowData
from .raw_client import AsyncRawSheetsClient, RawSheetsClient
from .types.update_sheet_range_request_values_item_item import UpdateSheetRangeRequestValuesItemItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SheetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSheetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSheetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSheetsClient
        """
        return self._raw_client

    def update_cell(
        self,
        *,
        asset_id: str,
        column: int,
        row: int,
        value: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Update a single cell in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        column : int
            1-based column index (e.g., 1 = column A)

        row : int
            1-based row index (e.g., 1 = first row)

        value : str
            Value to set in the cell

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.update_cell(
            asset_id="asset_id",
            column=1,
            row=1,
            value="value",
        )
        """
        _response = self._raw_client.update_cell(
            asset_id=asset_id, column=column, row=row, value=value, sheet_id=sheet_id, request_options=request_options
        )
        return _response.data

    def delete_cells(
        self,
        *,
        asset_id: str,
        end_column_index: int,
        end_row_index: int,
        start_column_index: int,
        start_row_index: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Delete cells from an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column_index : int
            1-based ending column index

        end_row_index : int
            1-based ending row index

        start_column_index : int
            1-based starting column index

        start_row_index : int
            1-based starting row index

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.delete_cells(
            asset_id="asset_id",
            end_column_index=1,
            end_row_index=1,
            start_column_index=1,
            start_row_index=1,
        )
        """
        _response = self._raw_client.delete_cells(
            asset_id=asset_id,
            end_column_index=end_column_index,
            end_row_index=end_row_index,
            start_column_index=start_column_index,
            start_row_index=start_row_index,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def delete_column(
        self,
        *,
        asset_id: str,
        column_indexes: typing.Sequence[int],
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Delete columns from an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        column_indexes : typing.Sequence[int]
            List of 1-based column indexes to delete

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.delete_column(
            asset_id="asset_id",
            column_indexes=[1],
        )
        """
        _response = self._raw_client.delete_column(
            asset_id=asset_id, column_indexes=column_indexes, sheet_id=sheet_id, request_options=request_options
        )
        return _response.data

    def insert_column(
        self,
        *,
        asset_id: str,
        reference_column_index: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Insert a column in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        reference_column_index : int
            1-based reference column index where to insert

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.insert_column(
            asset_id="asset_id",
            reference_column_index=1,
        )
        """
        _response = self._raw_client.insert_column(
            asset_id=asset_id,
            reference_column_index=reference_column_index,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def clear_formatting(
        self,
        *,
        asset_id: str,
        end_column_index: int,
        end_row_index: int,
        start_column_index: int,
        start_row_index: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Clear formatting from cells in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column_index : int
            1-based ending column index

        end_row_index : int
            1-based ending row index

        start_column_index : int
            1-based starting column index

        start_row_index : int
            1-based starting row index

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.clear_formatting(
            asset_id="asset_id",
            end_column_index=1,
            end_row_index=1,
            start_column_index=1,
            start_row_index=1,
        )
        """
        _response = self._raw_client.clear_formatting(
            asset_id=asset_id,
            end_column_index=end_column_index,
            end_row_index=end_row_index,
            start_column_index=start_column_index,
            start_row_index=start_row_index,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def clear_range(
        self,
        *,
        asset_id: str,
        num_columns: int,
        num_rows: int,
        start_column: int,
        start_row: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Clear a range of cells in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        num_columns : int
            Number of columns to clear

        num_rows : int
            Number of rows to clear

        start_column : int
            1-based starting column index

        start_row : int
            1-based starting row index

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.clear_range(
            asset_id="asset_id",
            num_columns=1,
            num_rows=1,
            start_column=1,
            start_row=1,
        )
        """
        _response = self._raw_client.clear_range(
            asset_id=asset_id,
            num_columns=num_columns,
            num_rows=num_rows,
            start_column=start_column,
            start_row=start_row,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def format_range(
        self,
        *,
        asset_id: str,
        end_column: int,
        end_row: int,
        formatting: CellFormat,
        start_column: int,
        start_row: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Apply formatting to a range of cells in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column : int
            1-based ending column index

        end_row : int
            1-based ending row index

        formatting : CellFormat
            Cell format

        start_column : int
            1-based starting column index

        start_row : int
            1-based starting row index

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena, CellFormat

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.format_range(
            asset_id="asset_id",
            end_column=1,
            end_row=1,
            formatting=CellFormat(),
            start_column=1,
            start_row=1,
        )
        """
        _response = self._raw_client.format_range(
            asset_id=asset_id,
            end_column=end_column,
            end_row=end_row,
            formatting=formatting,
            start_column=start_column,
            start_row=start_row,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def update_range(
        self,
        *,
        asset_id: str,
        start_column: int,
        start_row: int,
        values: typing.Sequence[typing.Sequence[typing.Optional[UpdateSheetRangeRequestValuesItemItem]]],
        formatting: typing.Optional[typing.Sequence[typing.Sequence[typing.Optional[CellFormat]]]] = OMIT,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Update a range of cells in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        start_column : int
            1-based starting column index

        start_row : int
            1-based starting row index

        values : typing.Sequence[typing.Sequence[typing.Optional[UpdateSheetRangeRequestValuesItemItem]]]
            2D list of cells for each row

        formatting : typing.Optional[typing.Sequence[typing.Sequence[typing.Optional[CellFormat]]]]
            Optional 2D list of cell formats matching the structure of values. Each row is a list of CellFormat objects for each cell in that row. Use None for cells without formatting. numberFormat is not required unless user explicity asked to change

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.update_range(
            asset_id="asset_id",
            start_column=1,
            start_row=1,
            values=[[]],
        )
        """
        _response = self._raw_client.update_range(
            asset_id=asset_id,
            start_column=start_column,
            start_row=start_row,
            values=values,
            formatting=formatting,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def insert_row(
        self,
        *,
        asset_id: str,
        reference_row_index: int,
        num_rows: typing.Optional[int] = OMIT,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Insert a row in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        reference_row_index : int
            1-based reference row index where to insert

        num_rows : typing.Optional[int]
            Number of rows to insert

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.insert_row(
            asset_id="asset_id",
            reference_row_index=1,
        )
        """
        _response = self._raw_client.insert_row(
            asset_id=asset_id,
            reference_row_index=reference_row_index,
            num_rows=num_rows,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def duplicate_sheet(
        self,
        *,
        asset_id: str,
        new_sheet_id: typing.Optional[int] = OMIT,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Duplicate an existing sheet in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        new_sheet_id : typing.Optional[int]
            New sheet ID for the duplicated sheet (auto-generated if not provided)

        sheet_id : typing.Optional[int]
            Sheet ID to duplicate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.duplicate_sheet(
            asset_id="asset_id",
        )
        """
        _response = self._raw_client.duplicate_sheet(
            asset_id=asset_id, new_sheet_id=new_sheet_id, sheet_id=sheet_id, request_options=request_options
        )
        return _response.data

    def create_tab(
        self, *, asset_id: str, sheet: Sheet, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateNewSheetTabResponse:
        """
        Create a new tab in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        sheet : Sheet
            Sheet Specification

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateNewSheetTabResponse
            Successful Response

        Examples
        --------
        from athena import Athena, Sheet

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.create_tab(
            asset_id="asset_id",
            sheet=Sheet(
                column_count=1,
                index=1,
                row_count=1,
                sheet_id=1,
                title="title",
            ),
        )
        """
        _response = self._raw_client.create_tab(asset_id=asset_id, sheet=sheet, request_options=request_options)
        return _response.data

    def delete_table_column(
        self,
        *,
        asset_id: str,
        dimension_index: int,
        table_id: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Delete a column from a table within an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        dimension_index : int
            0-based dimension index within the table

        table_id : str
            Table ID where to delete column

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.delete_table_column(
            asset_id="asset_id",
            dimension_index=1,
            table_id="table_id",
        )
        """
        _response = self._raw_client.delete_table_column(
            asset_id=asset_id,
            dimension_index=dimension_index,
            table_id=table_id,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def insert_table_column(
        self,
        *,
        asset_id: str,
        dimension_index: int,
        direction: str,
        table_id: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Insert a column in a table within an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        dimension_index : int
            0-based dimension index within the table

        direction : str
            Direction of insertion (left or right)

        table_id : str
            Table ID where to insert column

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.insert_table_column(
            asset_id="asset_id",
            dimension_index=1,
            direction="direction",
            table_id="table_id",
        )
        """
        _response = self._raw_client.insert_table_column(
            asset_id=asset_id,
            dimension_index=dimension_index,
            direction=direction,
            table_id=table_id,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def create_table(
        self,
        *,
        asset_id: str,
        end_column_index: int,
        end_row_index: int,
        start_column_index: int,
        start_row_index: int,
        table_id: str,
        table_name: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Create a table in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column_index : int
            1-based ending column index

        end_row_index : int
            1-based ending row index

        start_column_index : int
            1-based starting column index

        start_row_index : int
            1-based starting row index

        table_id : str
            Unique table ID

        table_name : str
            Name of the table

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.create_table(
            asset_id="asset_id",
            end_column_index=1,
            end_row_index=1,
            start_column_index=1,
            start_row_index=1,
            table_id="table_id",
            table_name="table_name",
        )
        """
        _response = self._raw_client.create_table(
            asset_id=asset_id,
            end_column_index=end_column_index,
            end_row_index=end_row_index,
            start_column_index=start_column_index,
            start_row_index=start_row_index,
            table_id=table_id,
            table_name=table_name,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    def get_table(
        self,
        *,
        asset_id: str,
        table_name: str,
        table_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetTableResponse:
        """
        Retrieve table data from an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        table_name : str
            Table name to retrieve

        table_id : typing.Optional[str]
            Table ID to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTableResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.get_table(
            asset_id="asset_id",
            table_name="table_name",
        )
        """
        _response = self._raw_client.get_table(
            asset_id=asset_id, table_name=table_name, table_id=table_id, request_options=request_options
        )
        return _response.data

    def insert_table_row(
        self,
        *,
        asset_id: str,
        row_data: typing.Sequence[TableRowData],
        table_name: str,
        table_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Insert rows into a table in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        row_data : typing.Sequence[TableRowData]
            Array of row objects where keys are column names and values are cell values

        table_name : str
            Table name to insert row into

        table_id : typing.Optional[str]
            Table ID to insert row into

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.insert_table_row(
            asset_id="asset_id",
            row_data=[{"key": "value"}],
            table_name="table_name",
        )
        """
        _response = self._raw_client.insert_table_row(
            asset_id=asset_id,
            row_data=row_data,
            table_name=table_name,
            table_id=table_id,
            request_options=request_options,
        )
        return _response.data

    def update_table(
        self,
        *,
        asset_id: str,
        end_column_index: int,
        end_row_index: int,
        start_column_index: int,
        start_row_index: int,
        table_id: str,
        table_name: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Update an existing table in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column_index : int
            1-based ending column index

        end_row_index : int
            1-based ending row index

        start_column_index : int
            1-based starting column index

        start_row_index : int
            1-based starting row index

        table_id : str
            Table ID to update

        table_name : str
            Name of the table

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        from athena import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.sheets.update_table(
            asset_id="asset_id",
            end_column_index=1,
            end_row_index=1,
            start_column_index=1,
            start_row_index=1,
            table_id="table_id",
            table_name="table_name",
        )
        """
        _response = self._raw_client.update_table(
            asset_id=asset_id,
            end_column_index=end_column_index,
            end_row_index=end_row_index,
            start_column_index=start_column_index,
            start_row_index=start_row_index,
            table_id=table_id,
            table_name=table_name,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data


class AsyncSheetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSheetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSheetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSheetsClient
        """
        return self._raw_client

    async def update_cell(
        self,
        *,
        asset_id: str,
        column: int,
        row: int,
        value: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Update a single cell in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        column : int
            1-based column index (e.g., 1 = column A)

        row : int
            1-based row index (e.g., 1 = first row)

        value : str
            Value to set in the cell

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.update_cell(
                asset_id="asset_id",
                column=1,
                row=1,
                value="value",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_cell(
            asset_id=asset_id, column=column, row=row, value=value, sheet_id=sheet_id, request_options=request_options
        )
        return _response.data

    async def delete_cells(
        self,
        *,
        asset_id: str,
        end_column_index: int,
        end_row_index: int,
        start_column_index: int,
        start_row_index: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Delete cells from an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column_index : int
            1-based ending column index

        end_row_index : int
            1-based ending row index

        start_column_index : int
            1-based starting column index

        start_row_index : int
            1-based starting row index

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.delete_cells(
                asset_id="asset_id",
                end_column_index=1,
                end_row_index=1,
                start_column_index=1,
                start_row_index=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_cells(
            asset_id=asset_id,
            end_column_index=end_column_index,
            end_row_index=end_row_index,
            start_column_index=start_column_index,
            start_row_index=start_row_index,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def delete_column(
        self,
        *,
        asset_id: str,
        column_indexes: typing.Sequence[int],
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Delete columns from an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        column_indexes : typing.Sequence[int]
            List of 1-based column indexes to delete

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.delete_column(
                asset_id="asset_id",
                column_indexes=[1],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_column(
            asset_id=asset_id, column_indexes=column_indexes, sheet_id=sheet_id, request_options=request_options
        )
        return _response.data

    async def insert_column(
        self,
        *,
        asset_id: str,
        reference_column_index: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Insert a column in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        reference_column_index : int
            1-based reference column index where to insert

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.insert_column(
                asset_id="asset_id",
                reference_column_index=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.insert_column(
            asset_id=asset_id,
            reference_column_index=reference_column_index,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def clear_formatting(
        self,
        *,
        asset_id: str,
        end_column_index: int,
        end_row_index: int,
        start_column_index: int,
        start_row_index: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Clear formatting from cells in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column_index : int
            1-based ending column index

        end_row_index : int
            1-based ending row index

        start_column_index : int
            1-based starting column index

        start_row_index : int
            1-based starting row index

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.clear_formatting(
                asset_id="asset_id",
                end_column_index=1,
                end_row_index=1,
                start_column_index=1,
                start_row_index=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.clear_formatting(
            asset_id=asset_id,
            end_column_index=end_column_index,
            end_row_index=end_row_index,
            start_column_index=start_column_index,
            start_row_index=start_row_index,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def clear_range(
        self,
        *,
        asset_id: str,
        num_columns: int,
        num_rows: int,
        start_column: int,
        start_row: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Clear a range of cells in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        num_columns : int
            Number of columns to clear

        num_rows : int
            Number of rows to clear

        start_column : int
            1-based starting column index

        start_row : int
            1-based starting row index

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.clear_range(
                asset_id="asset_id",
                num_columns=1,
                num_rows=1,
                start_column=1,
                start_row=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.clear_range(
            asset_id=asset_id,
            num_columns=num_columns,
            num_rows=num_rows,
            start_column=start_column,
            start_row=start_row,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def format_range(
        self,
        *,
        asset_id: str,
        end_column: int,
        end_row: int,
        formatting: CellFormat,
        start_column: int,
        start_row: int,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Apply formatting to a range of cells in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column : int
            1-based ending column index

        end_row : int
            1-based ending row index

        formatting : CellFormat
            Cell format

        start_column : int
            1-based starting column index

        start_row : int
            1-based starting row index

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena, CellFormat

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.format_range(
                asset_id="asset_id",
                end_column=1,
                end_row=1,
                formatting=CellFormat(),
                start_column=1,
                start_row=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.format_range(
            asset_id=asset_id,
            end_column=end_column,
            end_row=end_row,
            formatting=formatting,
            start_column=start_column,
            start_row=start_row,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def update_range(
        self,
        *,
        asset_id: str,
        start_column: int,
        start_row: int,
        values: typing.Sequence[typing.Sequence[typing.Optional[UpdateSheetRangeRequestValuesItemItem]]],
        formatting: typing.Optional[typing.Sequence[typing.Sequence[typing.Optional[CellFormat]]]] = OMIT,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Update a range of cells in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        start_column : int
            1-based starting column index

        start_row : int
            1-based starting row index

        values : typing.Sequence[typing.Sequence[typing.Optional[UpdateSheetRangeRequestValuesItemItem]]]
            2D list of cells for each row

        formatting : typing.Optional[typing.Sequence[typing.Sequence[typing.Optional[CellFormat]]]]
            Optional 2D list of cell formats matching the structure of values. Each row is a list of CellFormat objects for each cell in that row. Use None for cells without formatting. numberFormat is not required unless user explicity asked to change

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.update_range(
                asset_id="asset_id",
                start_column=1,
                start_row=1,
                values=[[]],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_range(
            asset_id=asset_id,
            start_column=start_column,
            start_row=start_row,
            values=values,
            formatting=formatting,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def insert_row(
        self,
        *,
        asset_id: str,
        reference_row_index: int,
        num_rows: typing.Optional[int] = OMIT,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Insert a row in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        reference_row_index : int
            1-based reference row index where to insert

        num_rows : typing.Optional[int]
            Number of rows to insert

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.insert_row(
                asset_id="asset_id",
                reference_row_index=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.insert_row(
            asset_id=asset_id,
            reference_row_index=reference_row_index,
            num_rows=num_rows,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def duplicate_sheet(
        self,
        *,
        asset_id: str,
        new_sheet_id: typing.Optional[int] = OMIT,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Duplicate an existing sheet in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        new_sheet_id : typing.Optional[int]
            New sheet ID for the duplicated sheet (auto-generated if not provided)

        sheet_id : typing.Optional[int]
            Sheet ID to duplicate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.duplicate_sheet(
                asset_id="asset_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.duplicate_sheet(
            asset_id=asset_id, new_sheet_id=new_sheet_id, sheet_id=sheet_id, request_options=request_options
        )
        return _response.data

    async def create_tab(
        self, *, asset_id: str, sheet: Sheet, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateNewSheetTabResponse:
        """
        Create a new tab in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        sheet : Sheet
            Sheet Specification

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateNewSheetTabResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena, Sheet

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.create_tab(
                asset_id="asset_id",
                sheet=Sheet(
                    column_count=1,
                    index=1,
                    row_count=1,
                    sheet_id=1,
                    title="title",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_tab(asset_id=asset_id, sheet=sheet, request_options=request_options)
        return _response.data

    async def delete_table_column(
        self,
        *,
        asset_id: str,
        dimension_index: int,
        table_id: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Delete a column from a table within an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        dimension_index : int
            0-based dimension index within the table

        table_id : str
            Table ID where to delete column

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.delete_table_column(
                asset_id="asset_id",
                dimension_index=1,
                table_id="table_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_table_column(
            asset_id=asset_id,
            dimension_index=dimension_index,
            table_id=table_id,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def insert_table_column(
        self,
        *,
        asset_id: str,
        dimension_index: int,
        direction: str,
        table_id: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Insert a column in a table within an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        dimension_index : int
            0-based dimension index within the table

        direction : str
            Direction of insertion (left or right)

        table_id : str
            Table ID where to insert column

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.insert_table_column(
                asset_id="asset_id",
                dimension_index=1,
                direction="direction",
                table_id="table_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.insert_table_column(
            asset_id=asset_id,
            dimension_index=dimension_index,
            direction=direction,
            table_id=table_id,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def create_table(
        self,
        *,
        asset_id: str,
        end_column_index: int,
        end_row_index: int,
        start_column_index: int,
        start_row_index: int,
        table_id: str,
        table_name: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Create a table in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column_index : int
            1-based ending column index

        end_row_index : int
            1-based ending row index

        start_column_index : int
            1-based starting column index

        start_row_index : int
            1-based starting row index

        table_id : str
            Unique table ID

        table_name : str
            Name of the table

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.create_table(
                asset_id="asset_id",
                end_column_index=1,
                end_row_index=1,
                start_column_index=1,
                start_row_index=1,
                table_id="table_id",
                table_name="table_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_table(
            asset_id=asset_id,
            end_column_index=end_column_index,
            end_row_index=end_row_index,
            start_column_index=start_column_index,
            start_row_index=start_row_index,
            table_id=table_id,
            table_name=table_name,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data

    async def get_table(
        self,
        *,
        asset_id: str,
        table_name: str,
        table_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetTableResponse:
        """
        Retrieve table data from an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        table_name : str
            Table name to retrieve

        table_id : typing.Optional[str]
            Table ID to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTableResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.get_table(
                asset_id="asset_id",
                table_name="table_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_table(
            asset_id=asset_id, table_name=table_name, table_id=table_id, request_options=request_options
        )
        return _response.data

    async def insert_table_row(
        self,
        *,
        asset_id: str,
        row_data: typing.Sequence[TableRowData],
        table_name: str,
        table_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Insert rows into a table in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        row_data : typing.Sequence[TableRowData]
            Array of row objects where keys are column names and values are cell values

        table_name : str
            Table name to insert row into

        table_id : typing.Optional[str]
            Table ID to insert row into

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.insert_table_row(
                asset_id="asset_id",
                row_data=[{"key": "value"}],
                table_name="table_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.insert_table_row(
            asset_id=asset_id,
            row_data=row_data,
            table_name=table_name,
            table_id=table_id,
            request_options=request_options,
        )
        return _response.data

    async def update_table(
        self,
        *,
        asset_id: str,
        end_column_index: int,
        end_row_index: int,
        start_column_index: int,
        start_row_index: int,
        table_id: str,
        table_name: str,
        sheet_id: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SheetOperationResponse:
        """
        Update an existing table in an Athena spreadsheet.

        Parameters
        ----------
        asset_id : str
            The ID of the spreadsheet asset

        end_column_index : int
            1-based ending column index

        end_row_index : int
            1-based ending row index

        start_column_index : int
            1-based starting column index

        start_row_index : int
            1-based starting row index

        table_id : str
            Table ID to update

        table_name : str
            Name of the table

        sheet_id : typing.Optional[int]
            Sheet ID (defaults to 1)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SheetOperationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.sheets.update_table(
                asset_id="asset_id",
                end_column_index=1,
                end_row_index=1,
                start_column_index=1,
                start_row_index=1,
                table_id="table_id",
                table_name="table_name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_table(
            asset_id=asset_id,
            end_column_index=end_column_index,
            end_row_index=end_row_index,
            start_column_index=start_column_index,
            start_row_index=start_row_index,
            table_id=table_id,
            table_name=table_name,
            sheet_id=sheet_id,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AopAsyncExecuteResponseOut,
        AopExecuteRequestIn,
        AopExecuteResponseOut,
        AssetContentRequestOut,
        AssetNode,
        AssetScreenshotResponseOut,
        BorderModel,
        BorderStyle,
        BordersModel,
        CellFormat,
        CellFormatHorizontalAlignment,
        CellFormatVerticalAlignment,
        Chunk,
        ChunkContentItem,
        ChunkContentItem_ImageUrl,
        ChunkContentItem_Text,
        ChunkResult,
        ChunkResultChunkId,
        Content,
        ConversationAssetInfo,
        ConversationMessage,
        ConversationResult,
        CreatableAssetType,
        CreateAssetResponseOut,
        CreateNewSheetTabResponse,
        CreateProjectResponseOut,
        CustomAgentResponse,
        DataFrameRequestOut,
        DataFrameRequestOutColumnsItem,
        DataFrameRequestOutDataItemItem,
        DataFrameRequestOutIndexItem,
        DataFrameUnknownFormatError,
        DimensionProperties,
        DriveAgentResponse,
        FileTooLargeError,
        FolderResponse,
        GeneralAgentConfig,
        GeneralAgentConfigEnabledToolsItem,
        GeneralAgentRequest,
        GeneralAgentResponse,
        GeneralAgentResponseMessage,
        GeneralAgentResponseMessageKwargs,
        GetTableResponse,
        GridRange,
        Id,
        ImageUrlContent,
        InputMessage,
        InputMessageContentItem,
        InputMessageContentItem_ImageUrl,
        InputMessageContentItem_Text,
        NumberFormatModel,
        NumberFormatType,
        PaginatedAssetsOut,
        PromptMessage,
        PublicAssetOut,
        ResearchAgentResponse,
        SaveAssetRequestOut,
        Sheet,
        SheetOperationResponse,
        SqlAgentResponse,
        StructuredDataExtractorResponse,
        TableRowData,
        TextContent,
        TextFormatModel,
        Textrotation,
        ThemeColor,
        ThreadStatusResponseOut,
        Type,
        WrapStrategy,
    )
    from .errors import (
        BadRequestError,
        ContentTooLargeError,
        InternalServerError,
        NotFoundError,
        UnauthorizedError,
        UnprocessableEntityError,
        UnsupportedMediaTypeError,
    )
    from . import agents, aop, assets, query, threads, tools
    from .client import AsyncAthena, Athena
    from .environment import AthenaEnvironment
    from .query import QueryExecuteRequestDatabaseAssetIds
    from .tools import ToolsDataFrameRequestColumnsItem
    from .version import __version__
_dynamic_imports: typing.Dict[str, str] = {
    "AopAsyncExecuteResponseOut": ".types",
    "AopExecuteRequestIn": ".types",
    "AopExecuteResponseOut": ".types",
    "AssetContentRequestOut": ".types",
    "AssetNode": ".types",
    "AssetScreenshotResponseOut": ".types",
    "AsyncAthena": ".client",
    "Athena": ".client",
    "AthenaEnvironment": ".environment",
    "BadRequestError": ".errors",
    "BorderModel": ".types",
    "BorderStyle": ".types",
    "BordersModel": ".types",
    "CellFormat": ".types",
    "CellFormatHorizontalAlignment": ".types",
    "CellFormatVerticalAlignment": ".types",
    "Chunk": ".types",
    "ChunkContentItem": ".types",
    "ChunkContentItem_ImageUrl": ".types",
    "ChunkContentItem_Text": ".types",
    "ChunkResult": ".types",
    "ChunkResultChunkId": ".types",
    "Content": ".types",
    "ContentTooLargeError": ".errors",
    "ConversationAssetInfo": ".types",
    "ConversationMessage": ".types",
    "ConversationResult": ".types",
    "CreatableAssetType": ".types",
    "CreateAssetResponseOut": ".types",
    "CreateNewSheetTabResponse": ".types",
    "CreateProjectResponseOut": ".types",
    "CustomAgentResponse": ".types",
    "DataFrameRequestOut": ".types",
    "DataFrameRequestOutColumnsItem": ".types",
    "DataFrameRequestOutDataItemItem": ".types",
    "DataFrameRequestOutIndexItem": ".types",
    "DataFrameUnknownFormatError": ".types",
    "DimensionProperties": ".types",
    "DriveAgentResponse": ".types",
    "FileTooLargeError": ".types",
    "FolderResponse": ".types",
    "GeneralAgentConfig": ".types",
    "GeneralAgentConfigEnabledToolsItem": ".types",
    "GeneralAgentRequest": ".types",
    "GeneralAgentResponse": ".types",
    "GeneralAgentResponseMessage": ".types",
    "GeneralAgentResponseMessageKwargs": ".types",
    "GetTableResponse": ".types",
    "GridRange": ".types",
    "Id": ".types",
    "ImageUrlContent": ".types",
    "InputMessage": ".types",
    "InputMessageContentItem": ".types",
    "InputMessageContentItem_ImageUrl": ".types",
    "InputMessageContentItem_Text": ".types",
    "InternalServerError": ".errors",
    "NotFoundError": ".errors",
    "NumberFormatModel": ".types",
    "NumberFormatType": ".types",
    "PaginatedAssetsOut": ".types",
    "PromptMessage": ".types",
    "PublicAssetOut": ".types",
    "QueryExecuteRequestDatabaseAssetIds": ".query",
    "ResearchAgentResponse": ".types",
    "SaveAssetRequestOut": ".types",
    "Sheet": ".types",
    "SheetOperationResponse": ".types",
    "SqlAgentResponse": ".types",
    "StructuredDataExtractorResponse": ".types",
    "TableRowData": ".types",
    "TextContent": ".types",
    "TextFormatModel": ".types",
    "Textrotation": ".types",
    "ThemeColor": ".types",
    "ThreadStatusResponseOut": ".types",
    "ToolsDataFrameRequestColumnsItem": ".tools",
    "Type": ".types",
    "UnauthorizedError": ".errors",
    "UnprocessableEntityError": ".errors",
    "UnsupportedMediaTypeError": ".errors",
    "WrapStrategy": ".types",
    "__version__": ".version",
    "agents": ".agents",
    "aop": ".aop",
    "assets": ".assets",
    "query": ".query",
    "threads": ".threads",
    "tools": ".tools",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AopAsyncExecuteResponseOut",
    "AopExecuteRequestIn",
    "AopExecuteResponseOut",
    "AssetContentRequestOut",
    "AssetNode",
    "AssetScreenshotResponseOut",
    "AsyncAthena",
    "Athena",
    "AthenaEnvironment",
    "BadRequestError",
    "BorderModel",
    "BorderStyle",
    "BordersModel",
    "CellFormat",
    "CellFormatHorizontalAlignment",
    "CellFormatVerticalAlignment",
    "Chunk",
    "ChunkContentItem",
    "ChunkContentItem_ImageUrl",
    "ChunkContentItem_Text",
    "ChunkResult",
    "ChunkResultChunkId",
    "Content",
    "ContentTooLargeError",
    "ConversationAssetInfo",
    "ConversationMessage",
    "ConversationResult",
    "CreatableAssetType",
    "CreateAssetResponseOut",
    "CreateNewSheetTabResponse",
    "CreateProjectResponseOut",
    "CustomAgentResponse",
    "DataFrameRequestOut",
    "DataFrameRequestOutColumnsItem",
    "DataFrameRequestOutDataItemItem",
    "DataFrameRequestOutIndexItem",
    "DataFrameUnknownFormatError",
    "DimensionProperties",
    "DriveAgentResponse",
    "FileTooLargeError",
    "FolderResponse",
    "GeneralAgentConfig",
    "GeneralAgentConfigEnabledToolsItem",
    "GeneralAgentRequest",
    "GeneralAgentResponse",
    "GeneralAgentResponseMessage",
    "GeneralAgentResponseMessageKwargs",
    "GetTableResponse",
    "GridRange",
    "Id",
    "ImageUrlContent",
    "InputMessage",
    "InputMessageContentItem",
    "InputMessageContentItem_ImageUrl",
    "InputMessageContentItem_Text",
    "InternalServerError",
    "NotFoundError",
    "NumberFormatModel",
    "NumberFormatType",
    "PaginatedAssetsOut",
    "PromptMessage",
    "PublicAssetOut",
    "QueryExecuteRequestDatabaseAssetIds",
    "ResearchAgentResponse",
    "SaveAssetRequestOut",
    "Sheet",
    "SheetOperationResponse",
    "SqlAgentResponse",
    "StructuredDataExtractorResponse",
    "TableRowData",
    "TextContent",
    "TextFormatModel",
    "Textrotation",
    "ThemeColor",
    "ThreadStatusResponseOut",
    "ToolsDataFrameRequestColumnsItem",
    "Type",
    "UnauthorizedError",
    "UnprocessableEntityError",
    "UnsupportedMediaTypeError",
    "WrapStrategy",
    "__version__",
    "agents",
    "aop",
    "assets",
    "query",
    "threads",
    "tools",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .aop_async_execute_response_out import AopAsyncExecuteResponseOut
    from .aop_execute_request_in import AopExecuteRequestIn
    from .aop_execute_response_out import AopExecuteResponseOut
    from .asset_content_request_out import AssetContentRequestOut
    from .asset_node import AssetNode
    from .asset_screenshot_response_out import AssetScreenshotResponseOut
    from .border_model import BorderModel
    from .border_style import BorderStyle
    from .borders_model import BordersModel
    from .cell_format import CellFormat
    from .cell_format_horizontal_alignment import CellFormatHorizontalAlignment
    from .cell_format_vertical_alignment import CellFormatVerticalAlignment
    from .chunk import Chunk
    from .chunk_content_item import ChunkContentItem, ChunkContentItem_ImageUrl, ChunkContentItem_Text
    from .chunk_result import ChunkResult
    from .chunk_result_chunk_id import ChunkResultChunkId
    from .content import Content
    from .conversation_asset_info import ConversationAssetInfo
    from .conversation_message import ConversationMessage
    from .conversation_result import ConversationResult
    from .creatable_asset_type import CreatableAssetType
    from .create_asset_response_out import CreateAssetResponseOut
    from .create_new_sheet_tab_response import CreateNewSheetTabResponse
    from .create_project_response_out import CreateProjectResponseOut
    from .custom_agent_response import CustomAgentResponse
    from .data_frame_request_out import DataFrameRequestOut
    from .data_frame_request_out_columns_item import DataFrameRequestOutColumnsItem
    from .data_frame_request_out_data_item_item import DataFrameRequestOutDataItemItem
    from .data_frame_request_out_index_item import DataFrameRequestOutIndexItem
    from .data_frame_unknown_format_error import DataFrameUnknownFormatError
    from .dimension_properties import DimensionProperties
    from .drive_agent_response import DriveAgentResponse
    from .file_too_large_error import FileTooLargeError
    from .folder_response import FolderResponse
    from .general_agent_config import GeneralAgentConfig
    from .general_agent_config_enabled_tools_item import GeneralAgentConfigEnabledToolsItem
    from .general_agent_request import GeneralAgentRequest
    from .general_agent_response import GeneralAgentResponse
    from .general_agent_response_message import GeneralAgentResponseMessage
    from .general_agent_response_message_kwargs import GeneralAgentResponseMessageKwargs
    from .get_table_response import GetTableResponse
    from .grid_range import GridRange
    from .id import Id
    from .image_url_content import ImageUrlContent
    from .input_message import InputMessage
    from .input_message_content_item import (
        InputMessageContentItem,
        InputMessageContentItem_ImageUrl,
        InputMessageContentItem_Text,
    )
    from .number_format_model import NumberFormatModel
    from .number_format_type import NumberFormatType
    from .paginated_assets_out import PaginatedAssetsOut
    from .prompt_message import PromptMessage
    from .public_asset_out import PublicAssetOut
    from .research_agent_response import ResearchAgentResponse
    from .save_asset_request_out import SaveAssetRequestOut
    from .sheet import Sheet
    from .sheet_operation_response import SheetOperationResponse
    from .sql_agent_response import SqlAgentResponse
    from .structured_data_extractor_response import StructuredDataExtractorResponse
    from .table_row_data import TableRowData
    from .text_content import TextContent
    from .text_format_model import TextFormatModel
    from .textrotation import Textrotation
    from .theme_color import ThemeColor
    from .thread_status_response_out import ThreadStatusResponseOut
    from .type import Type
    from .wrap_strategy import WrapStrategy
_dynamic_imports: typing.Dict[str, str] = {
    "AopAsyncExecuteResponseOut": ".aop_async_execute_response_out",
    "AopExecuteRequestIn": ".aop_execute_request_in",
    "AopExecuteResponseOut": ".aop_execute_response_out",
    "AssetContentRequestOut": ".asset_content_request_out",
    "AssetNode": ".asset_node",
    "AssetScreenshotResponseOut": ".asset_screenshot_response_out",
    "BorderModel": ".border_model",
    "BorderStyle": ".border_style",
    "BordersModel": ".borders_model",
    "CellFormat": ".cell_format",
    "CellFormatHorizontalAlignment": ".cell_format_horizontal_alignment",
    "CellFormatVerticalAlignment": ".cell_format_vertical_alignment",
    "Chunk": ".chunk",
    "ChunkContentItem": ".chunk_content_item",
    "ChunkContentItem_ImageUrl": ".chunk_content_item",
    "ChunkContentItem_Text": ".chunk_content_item",
    "ChunkResult": ".chunk_result",
    "ChunkResultChunkId": ".chunk_result_chunk_id",
    "Content": ".content",
    "ConversationAssetInfo": ".conversation_asset_info",
    "ConversationMessage": ".conversation_message",
    "ConversationResult": ".conversation_result",
    "CreatableAssetType": ".creatable_asset_type",
    "CreateAssetResponseOut": ".create_asset_response_out",
    "CreateNewSheetTabResponse": ".create_new_sheet_tab_response",
    "CreateProjectResponseOut": ".create_project_response_out",
    "CustomAgentResponse": ".custom_agent_response",
    "DataFrameRequestOut": ".data_frame_request_out",
    "DataFrameRequestOutColumnsItem": ".data_frame_request_out_columns_item",
    "DataFrameRequestOutDataItemItem": ".data_frame_request_out_data_item_item",
    "DataFrameRequestOutIndexItem": ".data_frame_request_out_index_item",
    "DataFrameUnknownFormatError": ".data_frame_unknown_format_error",
    "DimensionProperties": ".dimension_properties",
    "DriveAgentResponse": ".drive_agent_response",
    "FileTooLargeError": ".file_too_large_error",
    "FolderResponse": ".folder_response",
    "GeneralAgentConfig": ".general_agent_config",
    "GeneralAgentConfigEnabledToolsItem": ".general_agent_config_enabled_tools_item",
    "GeneralAgentRequest": ".general_agent_request",
    "GeneralAgentResponse": ".general_agent_response",
    "GeneralAgentResponseMessage": ".general_agent_response_message",
    "GeneralAgentResponseMessageKwargs": ".general_agent_response_message_kwargs",
    "GetTableResponse": ".get_table_response",
    "GridRange": ".grid_range",
    "Id": ".id",
    "ImageUrlContent": ".image_url_content",
    "InputMessage": ".input_message",
    "InputMessageContentItem": ".input_message_content_item",
    "InputMessageContentItem_ImageUrl": ".input_message_content_item",
    "InputMessageContentItem_Text": ".input_message_content_item",
    "NumberFormatModel": ".number_format_model",
    "NumberFormatType": ".number_format_type",
    "PaginatedAssetsOut": ".paginated_assets_out",
    "PromptMessage": ".prompt_message",
    "PublicAssetOut": ".public_asset_out",
    "ResearchAgentResponse": ".research_agent_response",
    "SaveAssetRequestOut": ".save_asset_request_out",
    "Sheet": ".sheet",
    "SheetOperationResponse": ".sheet_operation_response",
    "SqlAgentResponse": ".sql_agent_response",
    "StructuredDataExtractorResponse": ".structured_data_extractor_response",
    "TableRowData": ".table_row_data",
    "TextContent": ".text_content",
    "TextFormatModel": ".text_format_model",
    "Textrotation": ".textrotation",
    "ThemeColor": ".theme_color",
    "ThreadStatusResponseOut": ".thread_status_response_out",
    "Type": ".type",
    "WrapStrategy": ".wrap_strategy",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AopAsyncExecuteResponseOut",
    "AopExecuteRequestIn",
    "AopExecuteResponseOut",
    "AssetContentRequestOut",
    "AssetNode",
    "AssetScreenshotResponseOut",
    "BorderModel",
    "BorderStyle",
    "BordersModel",
    "CellFormat",
    "CellFormatHorizontalAlignment",
    "CellFormatVerticalAlignment",
    "Chunk",
    "ChunkContentItem",
    "ChunkContentItem_ImageUrl",
    "ChunkContentItem_Text",
    "ChunkResult",
    "ChunkResultChunkId",
    "Content",
    "ConversationAssetInfo",
    "ConversationMessage",
    "ConversationResult",
    "CreatableAssetType",
    "CreateAssetResponseOut",
    "CreateNewSheetTabResponse",
    "CreateProjectResponseOut",
    "CustomAgentResponse",
    "DataFrameRequestOut",
    "DataFrameRequestOutColumnsItem",
    "DataFrameRequestOutDataItemItem",
    "DataFrameRequestOutIndexItem",
    "DataFrameUnknownFormatError",
    "DimensionProperties",
    "DriveAgentResponse",
    "FileTooLargeError",
    "FolderResponse",
    "GeneralAgentConfig",
    "GeneralAgentConfigEnabledToolsItem",
    "GeneralAgentRequest",
    "GeneralAgentResponse",
    "GeneralAgentResponseMessage",
    "GeneralAgentResponseMessageKwargs",
    "GetTableResponse",
    "GridRange",
    "Id",
    "ImageUrlContent",
    "InputMessage",
    "InputMessageContentItem",
    "InputMessageContentItem_ImageUrl",
    "InputMessageContentItem_Text",
    "NumberFormatModel",
    "NumberFormatType",
    "PaginatedAssetsOut",
    "PromptMessage",
    "PublicAssetOut",
    "ResearchAgentResponse",
    "SaveAssetRequestOut",
    "Sheet",
    "SheetOperationResponse",
    "SqlAgentResponse",
    "StructuredDataExtractorResponse",
    "TableRowData",
    "TextContent",
    "TextFormatModel",
    "Textrotation",
    "ThemeColor",
    "ThreadStatusResponseOut",
    "Type",
    "WrapStrategy",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .bad_request_error import BadRequestError
    from .content_too_large_error import ContentTooLargeError
    from .internal_server_error import InternalServerError
    from .not_found_error import NotFoundError
    from .unauthorized_error import UnauthorizedError
    from .unprocessable_entity_error import UnprocessableEntityError
    from .unsupported_media_type_error import UnsupportedMediaTypeError
_dynamic_imports: typing.Dict[str, str] = {
    "BadRequestError": ".bad_request_error",
    "ContentTooLargeError": ".content_too_large_error",
    "InternalServerError": ".internal_server_error",
    "NotFoundError": ".not_found_error",
    "UnauthorizedError": ".unauthorized_error",
    "UnprocessableEntityError": ".unprocessable_entity_error",
    "UnsupportedMediaTypeError": ".unsupported_media_type_error",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BadRequestError",
    "ContentTooLargeError",
    "InternalServerError",
    "NotFoundError",
    "UnauthorizedError",
    "UnprocessableEntityError",
    "UnsupportedMediaTypeError",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .data_frame_request_out_columns_item import DataFrameRequestOutColumnsItem
from .data_frame_request_out_data_item_item import DataFrameRequestOutDataItemItem
from .data_frame_request_out_index_item import DataFrameRequestOutIndexItem


class DataFrameRequestOut(UniversalBaseModel):
    """
    Response model with JSON dataframe representation.
    """

    columns: typing.List[typing.Optional[DataFrameRequestOutColumnsItem]]
    data: typing.List[typing.List[typing.Optional[DataFrameRequestOutDataItemItem]]]
    index: typing.Optional[typing.List[typing.Optional[DataFrameRequestOutIndexItem]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
